package org.nisum.controller;

import org.nisum.dto.InventoryDTO;
import org.nisum.model.Inventory;
import org.nisum.service.InventoryService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.MessageSource;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.util.List;
import java.util.Locale;

@RestController
@RequestMapping("/inventory")
public class InventoryController {

    @Autowired
    private InventoryService inventoryService;
    @Autowired
    private MessageSource messageSource;

    @PostMapping("/create")
    public Mono<ResponseEntity<String>> AddProductToInventory(@RequestBody InventoryDTO inventoryDTO, Locale locale){
       return inventoryService.createInventory(inventoryDTO)
                .map(savedInventory -> ResponseEntity.status(HttpStatus.CREATED)
                        .body(messageSource.getMessage("inventory.created.success",new Object[]{},locale)))
                .defaultIfEmpty(ResponseEntity
                        .status(HttpStatus.BAD_REQUEST)
                        .body(messageSource.getMessage("inventory.invalid.data",new Object[]{},locale)));

    }

    @GetMapping("/check/{productId}/{quantity}")
    public ResponseEntity<Boolean> checkProductAvailability(@PathVariable("productId") String productId, @PathVariable("quantity") int quantity){
        return inventoryService.checkProductAvailability(productId,quantity);
    }

//    http://localhost:8082/inventory/allRecords
    @GetMapping("/allRecords")
    public Flux<InventoryDTO> getAllInventories(){
        return inventoryService.getAllInventories();
    }

    @GetMapping("/sequential")
    public Mono<List<String>> getSequentialInventoryDetails() {
        return inventoryService.getSequentialInventoryDetails();
    }
    @GetMapping("/sequential1")
    public Mono<List<String>> getSequentialInventoryDetails1() {
        return inventoryService.getSequentialInventoryDetails();
    }

    @GetMapping("/parallel")
    public Mono<List<String>> getParallelInventoryDetails() {
        return inventoryService.getParallelInventoryDetails();
    }

}
