package org.nisum.service;

import org.nisum.dto.InventoryDTO;
import org.nisum.mapper.InventoryMapper;
import org.nisum.model.Inventory;
import org.nisum.repository.InventoryRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Optional;

@Service
public class InventoryService {

    private final InventoryMapper inventoryMapper;

    @Autowired
    private InventoryRepository inventoryRepository;

    @Autowired
    private WebClient.Builder webClientBuilder;
    Boolean status;

    public static final Logger log = LoggerFactory.getLogger(InventoryService.class);

    @Autowired
    public InventoryService(InventoryMapper inventoryMapper) {
        this.inventoryMapper = inventoryMapper;
    }

    public InventoryDTO getInventoryDTO(Inventory inventory){
        return inventoryMapper.toInventoryDTO(inventory);
    }

    public Inventory getInventory(InventoryDTO inventoryDTO){
        return inventoryMapper.toInventory(inventoryDTO);
    }

    public Mono<InventoryDTO> createInventory(InventoryDTO inventoryDTO) {
        Inventory inventory = inventoryMapper.toInventory(inventoryDTO);
        inventory.setTimeStamp(LocalDateTime.now());
        inventory.setInStock(true);
        return Mono.just(inventory).flatMap(inventoryRepository::save)
                .map(inventoryMapper::toInventoryDTO).doOnSuccess(inventoryDTO1 -> {
                    log.info("product with product id: "+inventoryDTO1.getProductId() + " added successfully to the inventory");
                });

    }

    public Flux<InventoryDTO> getAllInventories(){
        Flux<Inventory> inventoryFlux = inventoryRepository.findAll();
        return inventoryFlux.map(inventoryMapper::toInventoryDTO);
    }

    public ResponseEntity<Boolean> checkProductAvailability(String productId, int requiredQuantity){
        Flux<Inventory> inventory = inventoryRepository.findByProductId(productId);
        inventory.subscribe(inventory1 -> {
            log.info(inventory1.getProductId() + " : " + inventory1.getQuantity());
            log.info(String.valueOf(requiredQuantity));
            if(inventory1 != null){
                log.info("inside not null");

                if(inventory1.getQuantity() >= requiredQuantity){
                    log.info("inside true");
                   status = true;
                }else{
                    status = false;
                }
            }else {
                log.info("inside false");
               status = false;
            }
        });

        log.info(String.valueOf(status));
        return ResponseEntity.ok(status);


    }

    //sequential request-response model
    public Mono<List<String>> getSequentialInventoryDetails(){
        return webClientBuilder.baseUrl("https://dummyjson.com")
                .build()
                .get()
                .uri("/products/1")
                .retrieve()
                .bodyToMono(String.class)
                .flatMap(s ->
                    webClientBuilder.baseUrl("https://dummyjson.com")
                            .build()
                            .get()
                            .uri("/products/2")
                            .retrieve()
                            .bodyToMono(String.class)
                            .map(s1 -> List.of(s,s1)));

    }

    //parallel request response model
    public Mono<List<String>> getParallelInventoryDetails(){
        Mono<String> product1 =webClientBuilder.baseUrl("https://dummyjson.com")
                .build()
                .get()
                .uri("/products/1")
                .retrieve()
                .bodyToMono(String.class);
        Mono<String> product2 =webClientBuilder.baseUrl("https://dummyjson.com")
                .build()
                .get()
                .uri("/products/2")
                .retrieve()
                .bodyToMono(String.class);

        return Mono.zip(product1,product2)
                .map(objects -> List.of(objects.getT1(),objects.getT2()));
    }
}
