package org.nisum.service;

import lombok.var;
import org.nisum.exception.OrderNotFoundException;
import org.nisum.model.Order;
import org.nisum.repository.OrderRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.mongodb.core.ReactiveMongoTemplate;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.stereotype.Service;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.time.LocalDateTime;


@Service
public class OrderService {

    public static final Logger log = LoggerFactory.getLogger(OrderService.class);

    @Autowired
    private OrderRepository orderRepository;
//    private final KafkaTemplate<String,String> kafkaTemplate;

    private final WebClient webClient;

    @Autowired
    public OrderService(WebClient.Builder webClientBuilder) {
        this.webClient = webClientBuilder.baseUrl("http://localhost:8082/inventory").build();
    }

    public Mono<String> placeOrder(String productId, int quantity){
        log.info(String.valueOf(checkProductAvailability(productId,quantity)));
        return Mono.just("Something");
//        return checkProductAvailability(productId, quantity)
//                .flatMap(isAvailable -> {
//                    if(isAvailable){
//                        return createOrder(productId,quantity);
//                    }
//                    return Mono.just("Product is out of stock or insufficient quantity.");
//                });

    }

    private Mono<Boolean> checkProductAvailability(String productId, int quantity){
        return webClient.get()
                .uri("/check/{'"+productId+"'}/{'"+quantity+"'}")
                .retrieve()
                .bodyToMono(Boolean.class);
    }



    private Mono<String> createOrder(String productId, int quantity){
        Order order = new Order();
        if(productId == null){
            log.error("product id cannot be null");
        }
        order.setStatus("Placed");
        order.setTimeStamp(LocalDateTime.now());
        order.setProductId(productId);
        order.setQuantity(quantity);
        orderRepository.save(order).doOnSuccess(order1 -> {
            log.info("order with order id: " + order1.getId() + " placed successfully");
        });
        return Mono.just("Order placed successfully for product " + order.getProductId());

    }

    public Flux<Order> getAllOrders(int page, int size) {
        Pageable pageable = PageRequest.of(page,size);
        return  orderRepository.findAllBy(pageable);

    }

//    public Flux<Order> getOrders(){
//        return orderRepository.findAll();
//    }
//
//    public Mono<Order> getOrderById(String id) {
//        return orderRepository.findById(id);
//    }
}
